﻿Imports System.ComponentModel
Imports System.Globalization
Imports System.IO
Imports System.Net
Imports System.Threading
Imports System.Xml

Namespace CustomLocalization
    Public Class SmartResourceManager
        Implements INotifyPropertyChanged
        Private Shared ReadOnly Instances As New List(Of SmartResourceManager)

        Private Shared _resourceSet As Dictionary(Of String, String)

        Private Shared ReadOnly ResourceSets As New Dictionary(Of String, Dictionary(Of String, String))

        Private ReadOnly localeClient As New LocaleService.LocaleServiceClient

        Public Event CultureChangeError As EventHandler(Of CultureChangeErrorEventArgs)

        Private Shared isInitialized As Boolean

        Private Shared lockObject As New Object

        Public Property ResourceSet As Dictionary(Of String, String)
            Get
                Return _resourceSet
            End Get
            Set(ByVal value As Dictionary(Of String, String))
                _resourceSet = value

                ' Notify all instances
                For Each obj In Instances
                    obj.NotifyPropertyChanged("ResourceSet")
                Next obj
            End Set
        End Property

        Public Sub New()
            Instances.Add(Me)
            AddHandler localeClient.GetResourcesCompleted, AddressOf LocaleClientGetResourcesCompleted
        End Sub

        Public Sub Initialize(ByVal neutralLanguageFile As String, ByVal neutralLanguage As String)
            SyncLock lockObject
                If isInitialized Then
                    Return
                End If

                isInitialized = True
            End SyncLock

            If String.IsNullOrWhiteSpace(neutralLanguageFile) Then
                ' No neutral resources
                ChangeCulture(Thread.CurrentThread.CurrentUICulture)
            Else
                LoadNeutralResources(neutralLanguageFile, neutralLanguage)
            End If
        End Sub

        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged


        Private Sub LoadNeutralResources(ByVal neutralLanguageFile As String, ByVal neutralLanguage As String)
            If String.IsNullOrWhiteSpace(neutralLanguageFile) Then
                Return
            End If

            Try
                Dim client As New WebClient
                AddHandler client.DownloadStringCompleted, AddressOf ClientDownloadStringCompleted
                client.DownloadStringAsync(New Uri(neutralLanguageFile, UriKind.Relative), neutralLanguage)
            Catch e1 As Exception
                If Not neutralLanguage.Equals(Thread.CurrentThread.CurrentUICulture.Name) Then
                    ChangeCulture(Thread.CurrentThread.CurrentUICulture)
                End If

                Return
            End Try
        End Sub

        Private Sub ClientDownloadStringCompleted(ByVal sender As Object, ByVal e As DownloadStringCompletedEventArgs)
            If e.Error Is Nothing AndAlso e.Result IsNot Nothing Then
                Using sr As New StringReader(e.Result)
                    Using xmlReader As XmlReader = XmlReader.Create(sr)
                        Dim resources = ResxToDictionary(xmlReader)
                        ResourceSets.Add(e.UserState.ToString(), resources)
                        ResourceSet = resources

                        ' Now load the browsers active culture
                        If Not e.UserState.ToString().Equals(Thread.CurrentThread.CurrentUICulture.Name) Then
                            ChangeCulture(Thread.CurrentThread.CurrentUICulture)
                        End If
                    End Using
                End Using
            End If
        End Sub

        Private Shared Function ResxToDictionary(ByVal resx As XmlReader) As Dictionary(Of String, String)
            Dim result As New Dictionary(Of String, String)
            Dim keyname = String.Empty

            Do While resx.Read()
                If resx.IsStartElement() AndAlso
                    resx.Name.Equals("value") AndAlso
                    (Not String.IsNullOrWhiteSpace(keyname)) Then
                    Dim val = resx.ReadElementContentAsString()
                    result.Add(keyname, val)
                    keyname = String.Empty
                End If

                If resx.IsStartElement() AndAlso resx.Name.Equals("data") Then
                    If resx.GetAttribute("name") IsNot Nothing Then
                        keyname = resx.GetAttribute("name")
                    End If
                End If
            Loop
            Return result
        End Function

        Private Sub LocaleClientGetResourcesCompleted(ByVal sender As Object, ByVal e As LocaleService.GetResourcesCompletedEventArgs)
            If e.Error IsNot Nothing Then
                Dim evt = CultureChangeErrorEvent

                If evt IsNot Nothing Then
                    evt(Me, New CultureChangeErrorEventArgs(TryCast(e.UserState, CultureInfo), e.Error))
                End If
            Else
                If e.Result Is Nothing Then
                    Return
                End If

                Dim culture = TryCast(e.UserState, CultureInfo)

                If culture Is Nothing Then
                    Return
                End If

                ResourceSets.Add(culture.Name, e.Result)
                ResourceSet = e.Result

                Thread.CurrentThread.CurrentUICulture = culture
                Thread.CurrentThread.CurrentCulture = Thread.CurrentThread.CurrentUICulture
            End If
        End Sub

        Public Sub NotifyPropertyChanged(ByVal [property] As String)
            Dim evt = PropertyChangedEvent

            If evt IsNot Nothing Then
                evt(Me, New PropertyChangedEventArgs([property]))
            End If
        End Sub

        Public Sub ChangeCulture(ByVal culture As CultureInfo)
            If Not ResourceSets.ContainsKey(culture.Name) Then
                localeClient.GetResourcesAsync(culture.Name, culture)
            Else
                ResourceSet = ResourceSets(culture.Name)

                Thread.CurrentThread.CurrentUICulture = culture
                Thread.CurrentThread.CurrentCulture = Thread.CurrentThread.CurrentUICulture
            End If
        End Sub

        Public Function GetString(ByVal key As String) As String
            If String.IsNullOrEmpty(key) Then
                Return String.Empty
            End If

            If _resourceSet.ContainsKey(key) Then
                Return _resourceSet(key)
            Else
                Return String.Empty
            End If
        End Function

        Public Function GetString(ByVal key As String, ByVal culture As String) As String
            If ResourceSets.ContainsKey(culture) Then
                If ResourceSets(culture).ContainsKey(key) Then
                    Return ResourceSets(culture)(key)
                Else
                    Return String.Empty
                End If
            Else
                Return String.Empty
            End If
        End Function
    End Class
End Namespace